//-------------------------------------------------------------------------------------------------
// Image.hpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
// 
// An Image is a generic representation of a 2d array of pixel data.
//	So far the Image class supports loading of the following formats:
//		- 24 bit uncompressed TGA
//		- 32 bit uncompressed TGA
//-------------------------------------------------------------------------------------------------
#ifndef _include_Image_
#define _include_Image_


//-------------------------------------------------------------------------------------------------
// Included header files
//-------------------------------------------------------------------------------------------------
#include "engineCommon.hpp"
#include "engineAssert.hpp"
#include "Rgba.hpp"


///////////////////////////////////////////////////////////////////////////////////////////////////
//
//		Image
//
///////////////////////////////////////////////////////////////////////////////////////////////////
class Image
{
private:
public:
	static const dword_t TEXNUM_UNREGISTERED;
	static byte_t	_primerTGA_correct[ 12 ];	// the first 12 bytes of a valid TGA file

	word_t			_width;						// number of pixels wide
	word_t			_height;					// number of pixels high
	byte_t			_bytesPerPixel;				// bytes per pixel; usually 3 or 4
	byte_t*			_pixelData;					// buffer containing ( _width * _height * _bytesPerPixel ) bytes
	GLenum			_internalFormat;			// format specification for how opengl should store the image internally
	GLenum			_bufferFormat;				// format specification for how opengl should read <_pixelData>
	std::string		_filePath;					// image file name path that was used for construction

	void _LoadFromFile( const std::string& filePath );
	void _LoadTGAFromFile( FILE* file );
	int _GetPixelOffset( int x, int y ) const;
	void _GetPixelCoordsFromByteOffset( int byteOffset, int& x, int& y ) const;

public:
	Image( const std::string& filePath );
	~Image();

	byte_t*			GetRawPixelData( void ) 	{ return _pixelData; }
	word_t			GetWidth( void ) const		{ return _width; }
	word_t			GetHeight( void ) const		{ return _height; }
	const Rgba		GetPixelColor( int x, int y ) const;
	void			SetPixelColor( int x, int y, const Rgba& rgba );
	void			SetPixelColor( int x, int y, byte_t redByte, byte_t greenByte, byte_t blueByte, byte_t alphaByte = (byte_t)0xff );

	friend class Texture;
};


//-------------------------------------------------------------------------------------------------
// GetPixelOffset
//-------------------------------------------------------------------------------------------------
inline int Image::_GetPixelOffset( int x, int y ) const
{
	ASSERT_MSG4( x >= 0 && x < _width && y >= 0 && y < _height, "Image::_GetPixelOffset() : coordinates (%d,%d) fall outside the image - must be in the range ( [0,%d], [0,%d] )!", x, y, _width-1, _height-1 );
	return( _bytesPerPixel * ( x + ((_height - y - 1) * _width) ) );
}


//-------------------------------------------------------------------------------------------------
// GetPixelCoordsFromByteOffset
//-------------------------------------------------------------------------------------------------
inline void Image::_GetPixelCoordsFromByteOffset( int byteOffset, int& x, int& y ) const
{
	int pixelNumber = byteOffset / _bytesPerPixel;
	x = pixelNumber % _width;
	y = pixelNumber / _width;
	y = _height - y - 1;
}



#endif // _include_Image_
