//-------------------------------------------------------------------------------------------------
// ParseFile.cpp (engine)
// 
// Copyright 2004 ALGDS, Zero Lodge.  See Engine/License.hpp for usage and license details.
//-------------------------------------------------------------------------------------------------
#include "pch.hpp"
#include "ParseFile.hpp"
#include "engineAssert.hpp"


//-------------------------------------------------------------------------------------------------
// Class constants
//-------------------------------------------------------------------------------------------------


//-------------------------------------------------------------------------------------------------
// Constructors
//-------------------------------------------------------------------------------------------------
ParseFile::ParseFile( const char* filePath )
:
_parseBuffer( NULL ),
_filePath( filePath ),
_fileHandle( NULL )
//_ownsFileHandle( true )
{
	VALIDATE_MSG0( filePath, "ParseFile::ParseFile() : <filePath> was NULL!" );

	/// Attempt to open the specified file
	_OpenFileByName( filePath );

	/// Read file contents into <parseBuffer>
	_LoadFileIntoBufferAndClose( true );
}


ParseFile::ParseFile( const std::string& filePath )
:
_parseBuffer( NULL ),
_filePath( filePath.c_str() ),
_fileHandle( NULL )
//_ownsFileHandle( true )
{

	/// Attempt to open the specified file
	_OpenFileByName( filePath.c_str() );

	/// Read file contents into <parseBuffer>
	_LoadFileIntoBufferAndClose( true );
}


ParseFile::ParseFile( FILE* fileHandle )
:
_parseBuffer( NULL ),
_filePath( "unknown" ),
_fileHandle( fileHandle )
//_ownsFileHandle( false )
{
	VALIDATE_MSG0( fileHandle, "ParseFile::ParseFile() : <fileHandle> was NULL!" );

	/// Read file contents into <parseBuffer>
	_LoadFileIntoBufferAndClose( false );
}


//-------------------------------------------------------------------------------------------------
// Destructor
//-------------------------------------------------------------------------------------------------
ParseFile::~ParseFile()
{
	delete[] const_cast<char*>( _parseBuffer->_buffer );
	delete _parseBuffer;
}



//-------------------------------------------------------------------------------------------------
// OpenFileByName
//-------------------------------------------------------------------------------------------------
void ParseFile::_OpenFileByName( const char* filePath )
{
	ASSERT_MSG0( filePath, "ParseFile::_OpenFileByName() : <filePath> was NULL" );

	/// Open file for reading
	_fileHandle = fopen( filePath, "rb" );
	VALIDATE_MSG1( _fileHandle, "ParseFile::_OpenFileByName() : could not open file \"%s\" for reading", filePath );
}


//-------------------------------------------------------------------------------------------------
// LoadFileIntoBufferAndClose
//-------------------------------------------------------------------------------------------------
void ParseFile::_LoadFileIntoBufferAndClose( bool closeFileHandle )
{
	ASSERT_MSG0( _fileHandle, "ParseFile::_LoadFileIntoBuffer() : <fileHandle> was NULL" );

	/// Determine file size
	fseek( _fileHandle, 0, SEEK_END );
	int fileSize = ftell( _fileHandle );
	rewind( _fileHandle );

	/// Allocate memory for buffer
	char* byteBuffer = new char[ fileSize + 1 ];
	VALIDATE_MSG1( byteBuffer, "ParseFile::_LoadFileIntoBufferAndClose() : new char[ %d ] failed!", fileSize + 1 );

	/// Read file contents into buffer
	fread( byteBuffer, 1, fileSize, _fileHandle );
	byteBuffer[ fileSize ] = '\0';

	/// Close the file (if so ordered)
	if( closeFileHandle )
	{
		fclose( _fileHandle );
	}

	/// Allocate a ParseBuffer object
	_parseBuffer = new ParseBuffer( byteBuffer, fileSize, _filePath );
	VALIDATE( _parseBuffer );
}


//-------------------------------------------------------------------------------------------------
// Passthrough functions
//-------------------------------------------------------------------------------------------------
void ParseFile::SkipWhitespace()
{
	_parseBuffer->SkipWhitespace();
}

void ParseFile::StripComments()
{
	_parseBuffer->StripComments();
}

int ParseFile::ParseInt()
{
	return _parseBuffer->ParseInt();
}

float ParseFile::ParseFloat()
{
	return _parseBuffer->ParseFloat();
}

double ParseFile::ParseDouble()
{
	return _parseBuffer->ParseDouble();
}

void ParseFile::ParseQuotedString( char* destBuffer, int destBufferSize, bool allowLineWrap )
{
	_parseBuffer->ParseQuotedString( destBuffer, destBufferSize, allowLineWrap );
}

void ParseFile::ParseQuotedString( std::string& destBuffer, int destBufferMaxSize, bool allowLineWrap )
{
	_parseBuffer->ParseQuotedString( destBuffer, destBufferMaxSize, allowLineWrap );
}

void ParseFile::ParseContiguousString( char* destBuffer, int destBufferSize )
{
	_parseBuffer->ParseContiguousString( destBuffer, destBufferSize );
}

void ParseFile::ParseStringToEndOfLine( char* destBuffer, int destBufferSize )
{
	_parseBuffer->ParseStringToEndOfLine( destBuffer, destBufferSize );
}

Vector2 ParseFile::ParseVector2()
{
	return _parseBuffer->ParseVector2();
}

Vector3 ParseFile::ParseVector3()
{
	return _parseBuffer->ParseVector3();
}

Vector4 ParseFile::ParseVector4()
{
	return _parseBuffer->ParseVector4();
}

void ParseFile::ParseExpectedToken( char token )
{
	_parseBuffer->ParseExpectedToken( token );
}

void ParseFile::ParseExpectedToken( const char* token )
{
	_parseBuffer->ParseExpectedToken( token );
}

char ParseFile::AdvanceToNextToken()
{
	return _parseBuffer->AdvanceToNextToken();
}



